package com.capitalware.ufm.action.objects;

import java.io.File;
import java.util.ArrayList;
import java.util.StringTokenizer;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.handlers.StringUtilities;
import com.capitalware.ufm.handlers.Tokens;
import com.capitalware.ufm.objects.CWConstants;
import com.capitalware.ufm.objects.StartEnd;

/**
 * This class holds the values for the ReplaceText action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class ReplaceTextObject extends UFMFileObject
{
   public String     srcFilename = null;
   public String     toFile = null;
   public String     toDir = null;
   public String     findText = null;
   public String     replaceText = null;
   public StartEnd   row = new StartEnd();
   public StartEnd   column = new StartEnd();
   public boolean    createDir = false;

   /**
    * The constructor
    */
   public ReplaceTextObject()
   {
      super();
   }

   /**
    * Process tokens in Action Object's variables
    */
   public void performTokenization()
   {
      if (srcFilename != null)
         srcFilename   = Tokens.replace(srcFilename, paClass.actionProperties);
      if (toFile != null)
         toFile = Tokens.replace(toFile, paClass.actionProperties);
   }

   /**
    * Verify the values in the object.
    */
   public void checkObject()
   {
      checkAndCreateDirectory(createDir, toDir);
      
      if (srcFilename == null)
         throw new IllegalArgumentException("Error: You must specify a filename for the ReplaceText action.");

      if (toFile == null)
         throw new IllegalArgumentException("Error: You must specify a toFilename for the ReplaceText action.");

      if ( (findText == null) || ("".equals(findText)) )
         throw new IllegalArgumentException("Error: You must specify a find for the ReplaceText action.");

      if ( (replaceText == null) || ("".equals(replaceText)) )
         throw new IllegalArgumentException("Error: You must specify a replace for the ReplaceText action.");

      if (row.start < 1)
         throw new IllegalArgumentException("Error: You must specify a row start value that is greater then zero for the ReplaceText action.");

      if ( (row.end < -1) || (row.end == 0) )
         throw new IllegalArgumentException("Error: You must specify a row end value that is greater then zero for the ReplaceText action.");

      if (column.start < 1)
         throw new IllegalArgumentException("Error: You must specify a column start value that is greater then zero for the ReplaceText action.");

      if ( (column.end < -1) || (column.end == 0) )
         throw new IllegalArgumentException("Error: You must specify a column end value that is greater then zero for the ReplaceText action.");
   }

   /**
    * Output the info
    * @param logger
    */
   public void printValues()
   {
      UFM.logger.info("     Source File          = " + srcFilename);
      UFM.logger.info("     Target Directory     = " + toDir);
      UFM.logger.info("     Target File          = " + toFile);
      UFM.logger.info("     Find                 = " + findText);
      UFM.logger.info("     Replace              = " + replaceText);
      UFM.logger.info("     Rows                 = " + row.start+":"+(row.end == -1?"*":row.end));
      UFM.logger.info("     Columns              = " + column.start+":"+(column.end == -1?"*":column.end));
      UFM.logger.info("     Create Directory     = " + createDir);
   }
   
   /**
    * Split and convert value
    * @param value
    * @param start
    * @param end
    */
   public void convertRTValue(String value, StartEnd se)
   {
      if ( ("*".equals(value)) || ("1:*".equals(value)) )
      {
         se.start = 1;
         se.end = -1;
      }
      else
      {
         StringTokenizer st = new StringTokenizer(value,":");
         
         if (st.hasMoreTokens())
         {
            se.start = StringUtilities.stringToInt(st.nextToken(), 1);
            
            if (st.hasMoreTokens())
               se.end = StringUtilities.stringToInt(st.nextToken(), -1);
            else
               se.end = -1;
         }
         else
         {
            se.start = 1;
            se.end = -1;
         }
      }
   }
   
   /**
    * Convert the object to a string.
    * @return s
    */
   public ArrayList<String> toStringArray()
   {
      ArrayList<String> al = new ArrayList<String>();
      String file;
      String r = row.start+":"+(row.end == -1?"*":row.end);
      String c = column.start+":"+(column.end == -1?"*":column.end);

      if (srcFilename.indexOf(File.separator) != -1 )
      {
         int x = srcFilename.lastIndexOf(File.separator);

         String dir   = srcFilename.substring(0, x);
         String fname = srcFilename.substring(x+1);

         file = "<"+CWConstants.FILE_ELEMENT+" "+CWConstants.DIR_ATTRIBUTE+"=\""+dir+"\">"+fname+"</"+CWConstants.FILE_ELEMENT+">";
      }
      else
         file = "<"+CWConstants.FILE_ELEMENT+">"+srcFilename+"</"+CWConstants.FILE_ELEMENT+">";
      
      if (toDir == null)
         al.add("<"+CWConstants.REPLACE_TEXT_ELEMENT+" "+CWConstants.TO_FILE_ATTRIBUTE+"=\""+toFile+" "+CWConstants.ROWS_ATTRIBUTE+"=\""+r+"\" "+CWConstants.COLUMNS_ATTRIBUTE+"=\""+c+"\">"+LINE_SEPARATOR);
      else
         al.add("<"+CWConstants.REPLACE_TEXT_ELEMENT+" "+CWConstants.TO_DIR_ATTRIBUTE+"=\""+toDir+"\" "+CWConstants.TO_FILE_ATTRIBUTE+"=\""+toFile+" "+CWConstants.ROWS_ATTRIBUTE+"=\""+r+"\" "+CWConstants.COLUMNS_ATTRIBUTE+"=\""+c+"\">"+LINE_SEPARATOR);
      
      al.add("  <"+CWConstants.FIND_ELEMENT+">"+findText+"</"+CWConstants.FIND_ELEMENT+">"+LINE_SEPARATOR);
      al.add("  <"+CWConstants.REPLACE_ELEMENT+">"+replaceText+"</"+CWConstants.REPLACE_ELEMENT+">"+LINE_SEPARATOR);
      al.add("  "+file+LINE_SEPARATOR);
      al.add("</"+CWConstants.REPLACE_TEXT_ELEMENT+">"+LINE_SEPARATOR);
      
      return al;
   }
   
   /**
    * Retrieve the values in this action for the status update.
    * @return s
    */
   public String toStringForStatusUpdate(String event)
   {
      String s;
      String r = row.start+":"+(row.end == -1?"*":row.end);
      String c = column.start+":"+(column.end == -1?"*":column.end);

      s = CWConstants.REPLACE_TEXT_ELEMENT+", "+labelName+", "+CWConstants.REPLACE_TEXT_ELEMENT+", "+CWConstants.CREATE_DIR_ATTRIBUTE+"="+createDir+", srcFilename="+srcFilename+", "+CWConstants.TO_DIR_ATTRIBUTE+"="+toDir+", "+CWConstants.TO_FILE_ATTRIBUTE+"="+toFile+", "+CWConstants.ROWS_ATTRIBUTE+"="+r+", "+CWConstants.COLUMNS_ATTRIBUTE+"="+c+", "+CWConstants.FIND_ELEMENT+"="+findText+", "+CWConstants.REPLACE_ELEMENT+"="+replaceText+LINE_SEPARATOR;

      return s;
   }
}
