package com.capitalware.ufm.actions;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.jdom.JDOMException;

import com.ibm.mq.MQException;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.action.objects.MQReceiveObject;
import com.capitalware.ufm.handlers.InternalMessageQueue;
import com.capitalware.ufm.internal.messages.UpdateStatus;
import com.capitalware.ufm.mq.MQGetMsg;
import com.capitalware.ufm.mq.objects.MQObject;
import com.capitalware.ufm.objects.CWConstants;
import com.capitalware.ufm.objects.UFMException;
import com.capitalware.ufm.xml.MQXml;

/**
 * This class will handle the receive action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class MQReceiveAction extends UFMAction
{
   private MQReceiveObject    ro;
   private InternalMessageQueue       iq = null;

   /**
    * The constructor
    * @param ro
    */
   public MQReceiveAction(MQReceiveObject ro, InternalMessageQueue iq)
   {
      super(ro.actionName);
      this.ro = ro;
      this.iq = iq;
   }

   /**
    * Perform the work.
    * @return exitCode
    */
   public int doWork()
   {
      int exitCode = 0;
      SimpleDateFormat dateTimeFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
      ro.printValues();
      
      try
      {
         if (ro.mqXMLFile != null)
         {
            MQXml mqX = new MQXml(ro.qMgrName, ro.qObject, ro.CCDTFile);
            mqX.readXMLFile(ro.mqXMLFile);
            ro.mqo = mqX.getMQObject();
         }
         else
         {
            ro.mqo = new MQObject(ro.qMgrName, ro.qObject, ro.CCDTFile);
         }
         
         ro.mqo.checkQM();
         ro.mqo.dumpUserParms();

         if ( (iq != null) && (ro.processType == CWConstants.DAEMON) )
            iq.addMessage(new UpdateStatus("0", dateTimeFormat.format(new Date())+", 0, "+ro.toStringForStatusUpdate("Starting")));
         
         exitCode = handleConnectingToMQ();
      }
      catch (JDOMException e)
      {
         UFM.logger.fatal("JDOMException: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());;
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 99;
      }
      catch (Exception e)
      {
         UFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());;
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 99;
      }
      finally
      {
         if (!ro.onErrorFail)
            exitCode = 0; /* reset back to zero */
      }

      return exitCode;
   }

   /**
    * Handle connecting and opening the input queue
    * @return exitCode
    */
   private int handleConnectingToMQ()
   {
      int      exitCode = 0;
      boolean  goForever = true;
      MQGetMsg mqh = null;

      ShutdownThread st = new ShutdownThread(mqh, ro.processType);
      Runtime.getRuntime().addShutdownHook(st);

      while (goForever)
      {
         mqh = new MQGetMsg(ro.mqo);
         st.updateMQH(mqh);
         
         try
         {
            mqh.connect();
            
            exitCode = mqh.retrieveMsg(ro, iq);
         }
         catch (MQException e)
         {
            UFM.logger.fatal("MQException: " + e.getLocalizedMessage());
            if (e.getCause() != null)
               UFM.logger.fatal("MQException: " + e.getCause());
            UFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
            UFM.globalErrorText = e.getLocalizedMessage();
            exitCode = 99;
         }
         catch (UFMException e)
         {
            UFM.logger.fatal("UFMException: " + e.getLocalizedMessage());
            UFM.globalErrorText = e.getLocalizedMessage();
            exitCode = 99;
         }
         catch (Exception e)
         {
            UFM.logger.fatal("Exception: " + e.getLocalizedMessage());
            if (e.getCause() != null)
               UFM.logger.fatal(e.getCause().toString());;
            UFM.globalErrorText = e.getLocalizedMessage();
            exitCode = 99;
         }
         finally
         {
            if ( mqh != null)
               mqh.disconnect();
            
            if (exitCode == 0)
            {
               goForever = false;  // everything is good so exit.
            }
            else if (ro.onErrorFail)
            {
               goForever = false;  // exitCode is not 0 but we must fail
            }
            else
            {
               /* exitCode is not 0 but we must NOT fail - go again */
               exitCode = 0; 
               
               /* But first sleep for 60 seconds */
               try
               {
                  Thread.sleep(60000); // time in milliseconds
               }
               catch (InterruptedException ie) {}
            }
         }
      }
      
      st.hasDisconnected();
      
      return exitCode;
   }

   /**
    * This class will handle the shutdown.
    *
    */
   private static class ShutdownThread extends Thread
   {
      private MQGetMsg  mqh;
      private boolean   hasDisconnected = false;
      private char      processType = CWConstants.ONE_PASS;

      /**
       * The constructor
       * @param mqh
       */
      public ShutdownThread(MQGetMsg mqh, char processType)
      {
         this.mqh = mqh;
         this.processType = processType;
      }

      /**
       * Update with the latest children thread
       * @param mqh
       */
      public void updateMQH(MQGetMsg mqh)
      {
         this.mqh = mqh;
      }

      /**
       * Notify this hook that MQReceive action has disconnected from the queue manager.
       */
      public void hasDisconnected()
      {
         hasDisconnected = true;
      }

      /**
       * Do the work.
       */
      public void run()
      {
         UFM.logger.info("ReceiveAction: Shutdown requested.");

         if ( mqh != null)
         {
            mqh.stopIt();
            
            if (processType == CWConstants.DAEMON)
            {
               int i = 0;
               
               while (i < 80)
               {
                  if (hasDisconnected)
                     break;
                  
                  // it does a Get with wait for 2 seconds, so we need to wait for it.
                  try
                  {
                     Thread.sleep(25); // time in milliseconds
                  }
                  catch (InterruptedException ie) {}
                  i++;
               }
               
            }
         }
      }
   }
}
