package com.capitalware.ufm.actions;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Properties;

import javax.mail.Authenticator;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.NoSuchProviderException;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.jdom.JDOMException;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.action.objects.SendEmailObject;
import com.capitalware.ufm.handlers.InternalMessageQueue;
import com.capitalware.ufm.handlers.Tokens;
import com.capitalware.ufm.internal.messages.UpdateStatus;
import com.capitalware.ufm.xml.EmailXml;

/**
 * This class will handle the SendEmail action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class SendEmailAction extends UFMAction
{
   private     SendEmailObject   seo;
   private     EmailXml          exh;
   private     InternalMessageQueue   iq = null;

   /**
    * The constructor
    * @param seo
    */
   public SendEmailAction(SendEmailObject seo, InternalMessageQueue iq)
   {
      super(seo.actionName);
      this.seo = seo;
      this.iq = iq;
   }

   /**
    * Perform the work.
    * @return exitCode
    */
   public int doWork()
   {
      SimpleDateFormat  dateTimeFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
      int               exitCode = 0;
      Session           mailSession = null;
      Transport         transport = null;

      seo.printValues();
      
      exh = new EmailXml();
      try
      {
         exh.readXMLFile(seo.xmlFile);
         
         Properties props = new Properties();
         props.put("mail.transport.protocol", "smtp");
         props.put("mail.smtp.host", exh.getSmtpHost());
         props.put("mail.smtp.port", exh.getSmtpPort());
         
         if (exh.isSsl())
         {
            props.put("mail.smtp.starttls.enable", "true");
            props.put("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
            props.put("mail.smtp.socketFactory.fallback", "false");
         }
         
         if (exh.isAuth())
         {
            props.put("mail.smtp.auth", "true");
            Authenticator auth = new SMTPAuthenticator();
            mailSession = Session.getInstance(props, auth);
         }
         else
            mailSession = Session.getInstance(props, null);
         
         // uncomment for debugging info to stdout
         // mailSession.setDebug(true);
         
         transport = mailSession.getTransport();
         transport.connect();
         UFM.logger.info("Connected to " + exh.getSmtpHost()+ " on port # " + exh.getSmtpPort());
         
         MimeMessage message = new MimeMessage(mailSession);
         message.setSentDate(new Date());
         
         /*
          * Do token replacement on the subject field
          */
         String subject = Tokens.replace(seo.actionName, exh.getSubject(), seo.paClass.actionProperties);
         if (UFM.globalErrorText == null)
            subject = Tokens.replace(subject, new StringBuffer(""));
         else
            subject = Tokens.replace(subject, new StringBuffer(UFM.globalErrorText));
         message.setSubject(subject);
         
         /*
          * Do token replacement on the message text field
          */
         String msgText = Tokens.replace(seo.actionName, exh.getMessageText(), seo.paClass.actionProperties);
         if (UFM.globalErrorText == null)
            msgText = Tokens.replace(msgText, new StringBuffer(""));
         else
            msgText = Tokens.replace(msgText, new StringBuffer(UFM.globalErrorText));
         message.setContent(msgText, "text/plain");
         
         message.setFrom(new InternetAddress(exh.getFromEmailAddress()));

         /*
          * Set 'TO' address
          */
         ArrayList<String> toEmail = exh.getToAddress();
         InternetAddress[] addressTo = new InternetAddress[toEmail.size()];
         for (int i = 0; i < toEmail.size(); i++)
            addressTo[i] = new InternetAddress(toEmail.get(i));

         message.setRecipients(Message.RecipientType.TO, addressTo);
         
         transport.sendMessage(message, message.getRecipients(Message.RecipientType.TO));

         seo.msgCount++;

         UFM.logger.info("Sent email to "+exh.getToAddress().toString());

         if (iq != null)
            iq.addMessage(new UpdateStatus("0", dateTimeFormat.format(new Date())+", 0, "+seo.actionName+", "+seo.labelName+", SMTP_Command=Sent email to "+exh.getToAddress().toString()+"\n"));
      }
      catch (NoSuchProviderException e)
      {
         UFM.logger.fatal("NoSuchProviderException: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());;
         exitCode = 8;
      }
      catch (MessagingException e)
      {
         UFM.logger.fatal("MessagingException: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());;
         exitCode = 8;
      }
      catch (JDOMException e)
      {
         UFM.logger.fatal("JDOMException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }
      catch (IOException e)
      {
         UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }
      finally
      {
         try
         {
            if (transport != null)
               transport.close();
         }
         catch (MessagingException e)
         {
            UFM.logger.fatal("MessagingException: " + e.getLocalizedMessage());
            if (e.getCause() != null)
               UFM.logger.fatal(e.getCause().toString());;
            exitCode = 8;
         }
         UFM.logger.info("Disconnected." );

         if (!seo.onErrorFail)
            exitCode = 0; /* reset back to zero */
      }
      
      return exitCode;
   }

   /**
    * Class to setup UserID and Password for connection to SMTP host. 
    */
   private class SMTPAuthenticator extends javax.mail.Authenticator
   {
      public PasswordAuthentication getPasswordAuthentication()
      {
         return new PasswordAuthentication(exh.getSmtpUserID(), exh.getSmtpPassword());
      }
   }
}
