package com.capitalware.ufm.actions;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.action.objects.UnZipObject;
import com.capitalware.ufm.handlers.IO;
import com.capitalware.ufm.handlers.Tokens;

/**
 * This class will handle the UnZip action.
 * 
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class UnZipAction extends UFMAction
{
   private UnZipObject uzo;

   /**
    * The constructor
    * @param uzo
    */
   public UnZipAction(UnZipObject uzo)
   {
      super(uzo.actionName);
      this.uzo = uzo;
   }

   /**
    * Perform the work.
    * @return exitCode
    */
   public int doWork()
   {
      int exitCode = 0;
      ZipFile zipFile = null;
      InputStream inputStream = null;
      
      uzo.printValues();

      uzo.zipFilename = Tokens.replace(uzo.actionName, uzo.zipFilename, uzo.toDir, 1, uzo.paClass.actionProperties);

      File inputFile = new File(uzo.zipFilename);
      try
      {
         // Wrap the input file with a ZipFile to iterate through its contents
         zipFile = new ZipFile(inputFile);
         Enumeration<? extends ZipEntry> oEnum = zipFile.entries();
         while (oEnum.hasMoreElements())
         {
            ZipEntry zipEntry = oEnum.nextElement();
            File file = new File(uzo.toDir + File.separatorChar + zipEntry.getName());

            if (zipEntry.isDirectory())
            {
               // If the entry in the ZIP file is a directory then create the directory
               file.mkdirs();
            }
            else
            {
               // Check if the combined path actually exists
               File completePath = new File(IO.getPath(file.getAbsolutePath()));
               if (!(completePath.exists()))
               {
                  completePath.mkdirs();
               }
               
               // If the entry in the ZIP file is a file then write the file 
               // in the appropriate directory location (as it is in the ZIP file)
               inputStream = zipFile.getInputStream(zipEntry);
               write(inputStream, file);
            }
         }
         
         uzo.fileCount++;

         UFM.logger.info("Successfully created archive '" + uzo.zipFilename + "' from '" + uzo.toDir + "'");
      }
      catch (IOException e)
      {
         UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }
      finally
      {
         // Clean up the I/O
         try
         {
            if (zipFile != null)
               zipFile.close();
         }
         catch (IOException e)
         {
            UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            UFM.globalErrorText = e.getLocalizedMessage();
            exitCode = 8;
         }
         try
         {
            if (inputStream != null)
               inputStream.close();
         }
         catch (IOException e)
         {
            UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            UFM.globalErrorText = e.getLocalizedMessage();
            exitCode = 8;
         }

         if (!uzo.onErrorFail)
            exitCode = 0; /* reset back to zero */
      }

      return exitCode;
   }

   /**
    * Writes to the supplied file with the contents read from the supplied input
    * stream.
    * 
    * @param is
    *           The Source input stream from where the contents will be read to
    *           write to the file.
    * 
    * @param outputFile
    *           The file to which the contents from the input stream will be
    *           written to.
    * 
    * @throws IOException
    *            Any problems while reading from the input stream or writing to
    *            the file.
    */
   private static void write(InputStream is, File outputFile) throws IOException
   {
      BufferedInputStream bis = null;
      FileOutputStream fos = null;
      BufferedOutputStream bos = null;
      
      try 
      {
         bis = new BufferedInputStream(is);
         fos = new FileOutputStream(outputFile);
         bos = new BufferedOutputStream(fos);

         // write bytes
         int byteData;
         while ((byteData = bis.read()) != -1)
         {
            bos.write((byte) byteData);
         }
      }
      finally
      {
         // close all the open streams
         try
         {
            if (bos != null)
               bos.close();
         }
         catch (IOException e)
         {
            UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            UFM.globalErrorText = e.getLocalizedMessage();
         }
         
         try
         {
            if (fos != null)
               fos.close();
         }
         catch (IOException e)
         {
            UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            UFM.globalErrorText = e.getLocalizedMessage();
         }
         
         try
         {
            if (bis != null)
               bis.close();
         }
         catch (IOException e)
         {
            UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            UFM.globalErrorText = e.getLocalizedMessage();
         }
      }

   }
}
