package com.capitalware.ufm.mq;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.security.InvalidAlgorithmParameterException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertStore;
import java.security.cert.LDAPCertStoreParameters;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.mq.objects.QueueObject;
import com.capitalware.ufm.mq.objects.MQObject;
import com.capitalware.ufm.objects.UFMException;

import com.ibm.mq.MQDistributionList;
import com.ibm.mq.MQDistributionListItem;
import com.ibm.mq.MQException;
import com.ibm.mq.MQQueue;
import com.ibm.mq.MQQueueManager;
import com.ibm.mq.MQSecurityExit;
import com.ibm.mq.constants.CMQC;

/**
 * A base class to define common MQ handler methods.
 *
 * @author Roger Lacroix, Capitalware Inc.
 * @version 1.0
 * @license Apache 2 License
 */
public class MyMQ extends Thread
{
   protected   MQObject        mqo;
   protected   MQQueueManager  _qMgr = null;

   /**
    * The constructor
    * @param logger
    */
   public MyMQ(MQObject mqo)
   {
      super();
      this.mqo = mqo;
   }

   /**
    * Connect to MQ Server
    *
    * @throws MQException
    * @throws MalformedURLException
    * @throws ClassNotFoundException
    * @throws IllegalAccessException
    * @throws InstantiationException
    * @throws NoSuchAlgorithmException
    * @throws InvalidAlgorithmParameterException
    */
   @SuppressWarnings("deprecation")
   public void connect() throws MQException, MalformedURLException, ClassNotFoundException, InstantiationException, IllegalAccessException, InvalidAlgorithmParameterException, NoSuchAlgorithmException, Exception
   {
      UFM.logger.debug("Attempting to connect to queue manager '"+mqo.qMgrName+"'");
      MQException.log = null;
      boolean setUserIDPassword = true;

      if (mqo.channelName != null)
      {
         Hashtable<String,Object> mqht = new Hashtable<String,Object>();

         mqht.put(CMQC.CHANNEL_PROPERTY, mqo.channelName);
         mqht.put(CMQC.HOST_NAME_PROPERTY, mqo.hostname);
         mqht.put(CMQC.PORT_PROPERTY, new Integer(mqo.port));

         if (mqo.securityExit != null)
         {
            if ( ("biz.capitalware.mqausx.MQAUSXJ".equals(mqo.securityExit)) ||
                 ("biz.capitalware.mqausx.MQAUSXJ2EE".equals(mqo.securityExit)) )
            {
               mqht.put(CMQC.SECURITY_EXIT_PROPERTY, CWSX.getMQAUSXJ(mqo));
               
               /* Did we already pass the info to MQAUSXJ or MQAUSXJ2EE? */
               if (mqo.securityExitData != null)
                  setUserIDPassword = false;
               else if ( (mqo.userID != null) && (mqo.password != null) )
                  setUserIDPassword = false;
            }
            else
            {
               Class se = getSEClass(mqo.securityExit, mqo.securityExitPath);
               mqht.put(CMQC.SECURITY_EXIT_PROPERTY, (MQSecurityExit) se.newInstance());
            }
         }
         
         if (setUserIDPassword)
         {
            if (mqo.userID != null)
               mqht.put(CMQC.USER_ID_PROPERTY, mqo.userID);

            if (mqo.password != null)
               mqht.put(CMQC.PASSWORD_PROPERTY, mqo.password);
         }

         /*
          * SSL
          */
         if (mqo.cipherSuiteName != null)
            mqht.put(CMQC.SSL_CIPHER_SUITE_PROPERTY, mqo.cipherSuiteName);

         if (mqo.distinguishedName != null)
            mqht.put(CMQC.SSL_PEER_NAME_PROPERTY, mqo.distinguishedName);

         if (mqo.trustedStore != null)
            System.setProperty("javax.net.ssl.trustStore", mqo.trustedStore);

         if (mqo.trustedStorePasswd != null)
            System.setProperty("javax.net.ssl.trustStorePassword", mqo.trustedStorePasswd);

         if (mqo.keyStore != null)
            System.setProperty("javax.net.ssl.keyStore", mqo.keyStore);

         if (mqo.keyStorePasswd != null)
            System.setProperty("javax.net.ssl.keyStorePassword", mqo.keyStorePasswd);

         if ( ((mqo.ldapServer != null)) && (mqo.ldapServerPort != 0) )
         {
            LDAPCertStoreParameters ldapcsparms = new LDAPCertStoreParameters(mqo.ldapServer, mqo.ldapServerPort);
            CertStore cs = CertStore.getInstance("LDAP", ldapcsparms);
            ArrayList al = new ArrayList();
            al.add(cs);
            mqht.put(CMQC.SSL_CERT_STORE_PROPERTY, al);
         }
         
         Enumeration<String> names = mqht.keys();
         String key;
         while(names.hasMoreElements())
         {
            key = (String) names.nextElement();

            if (CMQC.PASSWORD_PROPERTY.equalsIgnoreCase(key))
            {
               String pwd = (String)mqht.get(key);
               UFM.logger.info("connect() " + key + " length is '" + (pwd == null?"null":pwd.length()+"") +"'");
            }
            else
               UFM.logger.info("connect() " + key + "='" + mqht.get(key)+"'");
         }

         _qMgr = new MQQueueManager(mqo.qMgrName, mqht);

         UFM.logger.info("Successfully connected to " + mqo.qMgrName + " using " + mqo.channelName + " and " + mqo.hostname+"("+mqo.port+")");
      }
      else if (mqo.CCDTFile != null)
      {
         String trustedStore = System.getenv("MQSSLKEYR");
         if (trustedStore != null)
            System.setProperty("javax.net.ssl.trustStore", trustedStore + ".jks");

         File file = new File(mqo.CCDTFile);
         URL url = file.toURL();

         if (mqo.userID != null)
         {
            Hashtable<String,Object> mqht = new Hashtable<String,Object>();
   
            mqht.put(CMQC.USER_ID_PROPERTY, mqo.userID);

            if (mqo.password != null)
               mqht.put(CMQC.PASSWORD_PROPERTY, mqo.password);
            
            _qMgr = new MQQueueManager(mqo.qMgrName, mqht, url);
            
            UFM.logger.info("Successfully connected to " + mqo.qMgrName + " using " + mqo.CCDTFile + " with UserID");
         }
         else
         {
            _qMgr = new MQQueueManager(mqo.qMgrName, url);

            UFM.logger.info("Successfully connected to " + mqo.qMgrName + " using " + mqo.CCDTFile);
         }
      }
      else
      {
         _qMgr = new MQQueueManager(mqo.qMgrName);

         UFM.logger.info("Successfully connected to " + mqo.qMgrName);
      }
   }

   /**
    *
    * @param clazzName
    * @param pathToJar
    * @return
    */
   private Class getSEClass(String clazzName, String pathToJar) throws MalformedURLException, ClassNotFoundException
   {
      Class cls = null;

      // Create a File object on the root of the directory containing the class file
      File file = new File(pathToJar);

      // Convert File to a URL
      URL url = file.toURL();
      URL[] urls = new URL[]
      {
         url
      };

      // Create a new class loader with the directory
      ClassLoader cl = new URLClassLoader(urls);

      // Load in the class; MyClass.class should be located in
      // the directory file:/c:/myclasses/com/mycompany
      cls = cl.loadClass(clazzName);

      return cls;
   }

   /**
    * Disconnect from the queue manager
    */
   public void disconnect()
   {
      try
      {
         if (_qMgr != null)
         {
            UFM.logger.debug("Attempting to disconnect from queue manager '"+mqo.qMgrName+"'");
            _qMgr.disconnect();
            UFM.logger.info("Successfully disconnected.");
            _qMgr = null;
         }
      }
      catch (MQException e)
      {
         UFM.logger.fatal("MQException: " + e.getMessage());
         UFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
      }
      catch (Exception e)
      {
         UFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
      }
   }

   /**
    * Open a temporary dynamic queue
    * @throws UFMException 
    */
   public MQQueue openTempDynQ() throws UFMException
   {
      String inputQName   = "SYSTEM.DEFAULT.MODEL.QUEUE";
      int openInputOptions  = CMQC.MQOO_INQUIRE + CMQC.MQOO_FAIL_IF_QUIESCING + CMQC.MQOO_INPUT_SHARED;
      String DynamicQName = "CAPITALWARE.UFM.*";
      MQQueue inQ = null;
      
      try
      {
         UFM.logger.debug("Attempting to create/open temporary dynamic queue '"+DynamicQName+"'");
         
         inQ = _qMgr.accessQueue( inputQName,
                                  openInputOptions,
                                  null,           // default q manager
                                  DynamicQName,   // dynamic q name
                                  null );

         UFM.logger.info("Successfully opened temporary dynamic queue: '" + inQ.name + "'");
      }
      catch (MQException e)
      {
         UFM.logger.fatal("MQException: " + e.getLocalizedMessage() + " : QName=" + inputQName + " : DynamicQName="+DynamicQName);
         UFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
         UFM.globalErrorText = e.getLocalizedMessage();
         inQ = null;
         throw new UFMException("openTempDynQ: MQCC=" + e.completionCode + " : MQRC="+e.reasonCode+" : QName=" + inputQName + " : DynamicQName="+DynamicQName);
      }
      catch (Exception e)
      {
         UFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         inQ = null;
      }

      return inQ;
   }

   /**
    * Open the input queue
    *
    * @param qObject
    * @throws UFMException 
    * @throws MQException
    */
   public MQQueue openInQ(QueueObject qObject) throws UFMException
   {
      MQQueue inQ = null;
      try
      {
         UFM.logger.debug("Attempting to open input queue '"+qObject.qName+"'");
         
         inQ = _qMgr.accessQueue( qObject.qName,
                                  CMQC.MQOO_INPUT_AS_Q_DEF + CMQC.MQOO_FAIL_IF_QUIESCING,
                                  null,
                                  null,
                                  null );
         UFM.logger.info("Successfully opened input queue: '" + qObject + "'");
      }
      catch (MQException e)
      {
         UFM.logger.fatal("MQException: " + e.getLocalizedMessage() + " : QObject=" + qObject);
         UFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
         UFM.globalErrorText = e.getLocalizedMessage();
         inQ = null;
         throw new UFMException("openInQ: MQCC=" + e.completionCode + " : MQRC="+e.reasonCode+ " : QObject=" + qObject);
      }
      catch (Exception e)
      {
         UFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         inQ = null;
      }

      return inQ;
   }

   /**
    * Open the output queue
    *
    * @param qObject
    * @param contextRequired
    * @throws UFMException 
    */
   public MQQueue openOutQ(QueueObject qObject, boolean contextRequired) throws UFMException
   {
      MQQueue outQ = null;
      int putOptions = CMQC.MQOO_OUTPUT + CMQC.MQOO_FAIL_IF_QUIESCING;
      
      if (contextRequired) 
         putOptions += CMQC.MQOO_SET_IDENTITY_CONTEXT;
      
      try
      {
         UFM.logger.debug("Attempting to open output queue '"+qObject.qName+"'");
         
         outQ = _qMgr.accessQueue( qObject.qName,
                                   putOptions,
                                   qObject.qMgrName,
                                   null,
                                   null );
         UFM.logger.info("Successfully opened output queue: '" + qObject + "'");
      }
      catch (MQException e)
      {
         UFM.logger.fatal("MQException: " + e.getLocalizedMessage() + " : QObject=" + qObject);
         UFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
         UFM.globalErrorText = e.getLocalizedMessage();
         outQ = null;
         throw new UFMException("openOutQ: MQCC=" + e.completionCode + " : MQRC="+e.reasonCode+ " : QObject=" + qObject);
      }
      catch (Exception e)
      {
         UFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         outQ = null;
      }

      return outQ;
   }

   /**
    * Open the queue for both input and output
    *
    * @param qObject
    * @throws UFMException
    * @throws MQException
    */
   public MQQueue openInOutQ(QueueObject qObject) throws UFMException
   {
      MQQueue inOutQ = null;
      try
      {
         UFM.logger.debug("Attempting to open queue '"+qObject.qName+"'");

         inOutQ = _qMgr.accessQueue( qObject.qName,
                                   CMQC.MQOO_OUTPUT + CMQC.MQOO_INPUT_AS_Q_DEF + CMQC.MQOO_FAIL_IF_QUIESCING,
                                   qObject.qMgrName,
                                   null,
                                   null );
         UFM.logger.info("Successfully opened queue: '" + qObject + "'");
      }
      catch (MQException e)
      {
         UFM.logger.fatal("MQException: " + e.getLocalizedMessage() + " : QObject=" + qObject);
         UFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
         UFM.globalErrorText = e.getLocalizedMessage();
         inOutQ = null;
         throw new UFMException("openOutQ: MQCC=" + e.completionCode + " : MQRC="+e.reasonCode+ " : QObject=" + qObject);
      }
      catch (Exception e)
      {
         UFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         inOutQ = null;
      }

      return inOutQ;
   }

   /**
    * Close the queue.
    *
    * @param qHandle
    * @throws UFMException 
    */
   public void closeQ(MQQueue qHandle)
   {
      try
      {
         if (qHandle != null)
         {
            String qName = qHandle.name.trim();
            
            UFM.logger.debug("Attempting to close queue '"+qName+"'");
            
            qHandle.close();
            UFM.logger.info("Successfully closed the queue '"+qName+"'");
         }
      }
      catch (MQException e)
      {
         UFM.logger.fatal("MQException: " + "MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
         UFM.logger.fatal("MQException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
      }
      catch (Exception e)
      {
         UFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
      }
   }

   /**
    * Open the output distribution list
    *
    * @param ArrayList<MQDistListObject> disList
    * @param contextRequired
    * @throws UFMException 
    */
   public MQDistributionList openOutDL(ArrayList<QueueObject> disList, boolean contextRequired) throws UFMException
   {
      MQDistributionList outDL = null;
      MQDistributionListItem[] mqdlis = new MQDistributionListItem[disList.size()];
      QueueObject mqdlo = null;
      int putOptions = CMQC.MQOO_OUTPUT + CMQC.MQOO_FAIL_IF_QUIESCING;
      
      if (contextRequired) 
         putOptions += CMQC.MQOO_SET_IDENTITY_CONTEXT;

      for (int i=0; i < disList.size(); i++)
      {
         mqdlo =  disList.get(i);
         mqdlis[i] = new MQDistributionListItem();
         mqdlis[i].queueName = mqdlo.qName;
         if (mqdlo.qMgrName != null)
            mqdlis[i].queueManagerName = mqdlo.qMgrName;
      }
      
      try
      {
         UFM.logger.debug("Attempting to open distribution list queue '"+disList.toString()+"'");
         
         outDL = _qMgr.accessDistributionList( mqdlis, putOptions);
         UFM.logger.info("Successfully opened distribution list of queues: '" + disList.toString() + "'");
      }
      catch (MQException e)
      {
         UFM.logger.fatal("MQException: " + e.getLocalizedMessage() + " : distribution list of queues '" + disList.toString()+"'");
         UFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
         UFM.globalErrorText = e.getLocalizedMessage();
         outDL = null;
         throw new UFMException("openOutQ: MQCC=" + e.completionCode + " : MQRC="+e.reasonCode+ " : distribution list of queues '" + disList.toString()+"'");
      }
      catch (Exception e)
      {
         UFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         outDL = null;
      }

      return outDL;
   }

   /**
    * Close the distribution list.
    *
    * @param dlHandle
    * @throws UFMException 
    */
   public void closeDL(MQDistributionList dlHandle)
   {
      try
      {
         if (dlHandle != null)
         {
            UFM.logger.debug("Attempting to close distribution list queue '"+mqo.disListOfQNames+"'");
            
            dlHandle.close();
            UFM.logger.info("Successfully closed the distribution list '"+mqo.disListOfQNames+"'");
         }
      }
      catch (MQException e)
      {
         UFM.logger.fatal("MQException: " + "MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
         UFM.logger.fatal("MQException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
      }
      catch (Exception e)
      {
         UFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
      }
   }

   /**
    * Handle backing out the message(s).
    */
   public void backout()
   {
      try
      {
         UFM.logger.debug("Issuing backout." );
         _qMgr.backout();
         UFM.logger.info("Issued backout." );
      }
      catch (MQException e)
      {
         UFM.logger.fatal("MQException: " + e.getMessage() + " : QObject=" + mqo.qObject);
         UFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC=" + e.reasonCode);
         UFM.globalErrorText = e.getLocalizedMessage();
      }
      catch (Exception e)
      {
         UFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
      }
   }
}
